name
pangolin(syncFetch: true) {
  runHash
  reportTimestamp
  version
  lineage
  probability
  status
  note
}
bestMatchingSubtype {
  display
  referenceAccession
}
availableGenes { name }
validationResults {
  level
  message
}
mixtureRate
maxMixtureRate
minPrevalence
actualMinPrevalence
minCodonReads
minPositionReads
assembledConsensus

readDepthStats {
  median: percentile(p: 50)
  p25: percentile(p: 25)
  p75: percentile(p: 75)
}

cutoffKeyPoints {
  mixtureRate
  minPrevalence
  isAboveMixtureRateThreshold
  isBelowMinPrevalenceThreshold
}

allGeneSequenceReads {
  firstAA
  lastAA
  unsequencedRegions {
    size
    regions {
      posStart
      posEnd
    }
  }
  gene {
    name
    length
  }
  mutations {
    reference
    position
    AAs
    isUnsequenced
    text
    allAAReads {
      aminoAcid
      numReads
      percent
    }
    totalReads
  }
}
